package xslice

import (
	"fmt"
	"strings"

	"github.com/jinzhu/copier"
	"github.com/spf13/cast"
	"gitlab.wanzhuangkj.com/tush/xpkg/utils/xsf"
	"gitlab.wanzhuangkj.com/tush/xpkg/xerrors/xerror"
	"gitlab.wanzhuangkj.com/tush/xpkg/xset"
)

// connector 连接符
func Join[T any](rs []T, connector string) string {
	if len(rs) == 0 {
		return ""
	}
	strs := make([]string, 0, len(rs))
	for _, r := range rs {
		v := cast.ToString(r)
		strs = append(strs, v)
	}
	return strings.Join(strs, connector)
}

func Contains[T comparable](rs []T, r T) bool {
	for _, v := range rs {
		if v == r {
			return true
		}
	}
	return false
}

func NotContains[T comparable](rs []T, r T) bool {
	return !Contains(rs, r)
}

func StrToUint64(rs []string) []uint64 {
	if len(rs) == 0 {
		return nil
	}
	var IDs []uint64
	for _, r := range rs {
		IDs = append(IDs, cast.ToUint64(r))
	}
	return IDs
}

type IID interface {
	GetID() xsf.ID
}

type IIDTable interface {
	GetID() xsf.ID
	TableName() string
}

type ITable interface {
	TableName() string
}

func GetIDs[T IID](rs []*T) []xsf.ID {
	if len(rs) == 0 {
		return nil
	}
	set := xset.NewSet[xsf.ID]()
	for _, r := range rs {
		set.Add((*r).GetID())
	}
	return set.ToList()
}

func SliceToIDMap[T IID](rs []*T) map[xsf.ID]*T {
	m := make(map[xsf.ID]*T)
	for _, r := range rs {
		m[(*r).GetID()] = r
	}
	return m
}

func SliceToIdMap[T IId](rs []*T) map[uint]*T {
	m := make(map[uint]*T)
	for _, r := range rs {
		m[(*r).GetID()] = r
	}
	return m
}

func MapToSlice[T comparable, FromT, ToT any](m map[T]*FromT) []*ToT {
	if len(m) == 0 {
		return nil
	}
	ret := make([]*ToT, len(m))
	for _, r := range m {
		ret = append(ret, TypeToType[FromT, ToT](r))
	}
	return ret
}

func MapToMap[T comparable, FromT, ToT any](m map[T]*FromT) map[T]*ToT {
	if len(m) == 0 {
		return nil
	}
	ret := make(map[T]*ToT, len(m))
	for k, r := range m {
		ret[k] = TypeToType[FromT, ToT](r)
	}
	return ret
}

func TypeToType[FromT, ToT any](r *FromT) *ToT {
	if r == nil {
		return nil
	}
	var ret ToT
	copier.Copy(&ret, r)
	return &ret
}

func SliceToSlice[FromT, ToT any](rs []*FromT) []*ToT {
	if len(rs) == 0 {
		return nil
	}
	ret := make([]*ToT, 0, len(rs))
	for _, r := range rs {
		if r == nil {
			continue
		}
		ret = append(ret, TypeToType[FromT, ToT](r))
	}
	return ret
}

func MapSliceToMapSlice[T comparable, FromT, ToT any](m map[T][]*FromT) map[T][]*ToT {
	if len(m) == 0 {
		return nil
	}
	ret := make(map[T][]*ToT, len(m))
	for k, rs := range m {
		ret[k] = SliceToSlice[FromT, ToT](rs)
	}
	return ret
}

func MapSliceToSlice[T comparable, FromT, ToT any](m map[T][]*FromT) []*ToT {
	if len(m) == 0 {
		return nil
	}
	ret := make([]*ToT, 0)
	for _, rs := range m {
		ret = append(ret, SliceToSlice[FromT, ToT](rs)...)
	}
	return ret
}

// SetDifference 计算两个切片的差集 A - B
func SetDifference[T comparable](A, B []T) []T {
	result := []T{}
	setB := make(map[T]struct{})
	for _, elem := range B {
		setB[elem] = struct{}{}
	}
	for _, elem := range A {
		if _, found := setB[elem]; !found {
			result = append(result, elem)
		}
	}
	return result
}

func StrMapToSlice[T any](m map[string]*T) []*T {
	var rs []*T
	for _, r := range m {
		rs = append(rs, r)
	}
	return rs
}

func GetStrMapIDs[T IID](m map[string]*T) []xsf.ID {
	st := xset.NewSet[xsf.ID]()
	for _, v := range m {
		st.Add((*v).GetID())
	}
	return st.ToList()
}

func IDMapToSlice[T any](m map[xsf.ID]*T) []*T {
	var rs []*T
	for _, r := range m {
		rs = append(rs, r)
	}
	return rs
}

func GetIDMapIDs[T IID](m map[xsf.ID]*T) []xsf.ID {
	st := xset.NewSet[xsf.ID]()
	for _, v := range m {
		st.Add((*v).GetID())
	}
	return st.ToList()
}

func StrMapSliceToSlice[T any](m map[string][]*T) []*T {
	var rs []*T
	for _, r := range m {
		rs = append(rs, r...)
	}
	return rs
}

func GetStrMapSliceIDs[T IID](m map[string][]*T) []xsf.ID {
	st := xset.NewSet[xsf.ID]()
	for _, rs := range m {
		for _, r := range rs {
			st.Add((*r).GetID())
		}
	}
	return st.ToList()
}

func IDMapSliceToSlice[T any](m map[xsf.ID][]*T) []*T {
	var rs []*T
	for _, r := range m {
		rs = append(rs, r...)
	}
	return rs
}

func GetIDMapSliceIDs[T IID](m map[xsf.ID][]*T) []xsf.ID {
	st := xset.NewSet[xsf.ID]()
	for _, rs := range m {
		for _, r := range rs {
			st.Add((*r).GetID())
		}
	}
	return st.ToList()
}

func CompareSlice[T IIDTable, BizSlice ~[]*T](ids []xsf.ID, bizSlice BizSlice, errCode int) error {
	if len(ids) == 0 {
		return nil
	}
	if len(bizSlice) < len(ids) {
		if len(bizSlice) == 0 {
			var t T
			return xerror.NewC(errCode, fmt.Sprintf("未查询到%s", t.TableName()))
		}
		deltaIDs := SetDifference(ids, GetIDs(bizSlice))
		if len(deltaIDs) > 0 {
			var t T
			return xerror.NewC(errCode, fmt.Sprintf("未查询到%s[ids:%s]", t.TableName(), Join(deltaIDs, ",")))
		}
	}
	return nil
}

func ArrayUnique[T comparable](array []T) []T {
	mp := make(map[T]struct{})
	idx := 0
	for _, value := range array {
		if _, ok := mp[value]; ok {
			continue
		}
		array[idx] = value
		idx = idx + 1
		mp[value] = struct{}{}
	}
	return array[:idx]
}

type IId interface {
	GetID() uint
}

func GetIds[T IId](rs []*T) []uint {
	if len(rs) == 0 {
		return nil
	}
	set := xset.NewSet[uint]()
	for _, r := range rs {
		set.Add((*r).GetID())
	}
	return set.ToList()
}
